package fit;

// Copyright (c) 2002 Cunningham & Cunningham, Inc.
// Released under the terms of the GNU General Public License version 2 or later.

import java.lang.reflect.*;

public class ColumnFixture extends Fixture {

    TypeAdapter columnBindings[];

    // Traversal ////////////////////////////////

    public void doRows(Parse rows) {
        bind(rows.parts);
        super.doRows(rows.more);
    }

    public void doCell(Parse cell, int column) {
        TypeAdapter a = columnBindings[column];
        try {
            if (a == null) {
                ignore(cell);
            } else if (a.field != null) {
                String text = cell.text();
                if (!text.equals("")) {
                    a.set(a.parse(text));
                }
            } else if (a.method != null) {
                check(cell, a);
            }
        } catch(Exception e) {
            exception(cell, e);
        }
    }

    // Utility //////////////////////////////////

    void bind (Parse heads) {
        columnBindings = new TypeAdapter[heads.size()];
        for (int i=0; heads!=null; i++, heads=heads.more) {
            String name = heads.text();
            String sufix = "()";
            try {
                if (name.endsWith(sufix)) {
                    columnBindings[i] = bindMethod(name.substring(0,name.length()-sufix.length()));
                } else {
                    columnBindings[i] = bindField(name);
                }
            }
            catch (Exception e) {
                exception (heads, e);
            }
        }

    }

    TypeAdapter bindMethod (String name) throws Exception {
        Method methods[] = getTargetClass().getMethods();
        for (int i=0; i<methods.length; i++) {
            Method m = methods[i];
            if (m.getParameterTypes().length==0 &&
                m.getName().equals(name)) {
                    return TypeAdapter.on(this, m);
            }
        }
        throw new NoSuchMethodException();
    }

    TypeAdapter bindField (String name) throws Exception {
        return TypeAdapter.on(this, getTargetClass().getField(name));
    }

    Class getTargetClass() {
        return getClass();
    }
}
